/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/
module mdyLcdDispDynaRect( 
    clk_50m     ,  
    rst_n       , 

    lcd_hsync   ,
    lcd_vsync   ,
    lcd_de      ,
 

    lcd_rgb     ,
    lcd_dclk    
       
);

    input               clk_50m     ;  
    input               rst_n       ; 
    output              lcd_hsync   ;
    output              lcd_vsync   ;
    output              lcd_de      ;

    output  [23:0]      lcd_rgb     ;
    output              lcd_dclk    ;
      

    reg      [30:0]        h        ;
	reg      [30:0]        v        ;
	 
    reg                 lcd_hsync   ;
    reg                 lcd_vsync   ;
 
    reg     [23:0]      lcd_rgb     ;


    parameter     LINE_PR   =  1056 ;       
    parameter     FRAME_PER =   525 ;           


    parameter     H_SYNC    =    20 ;       
    parameter     V_SYNC    =    10 ;              

    parameter     HDE_START =    46 ;
    parameter     HDE_END   =   846 ;
    parameter     VDE_START =    23 ;
    parameter     VDE_END   =   503 ;

    

    reg    [12:0]     cnt_hsy       ;
    reg    [12:0]     cnt_vsy       ;
    reg               hsync_de      ;
    reg               vsync_de      ;

    wire              display_area  ;
    wire              e_area        ;
    wire              add_cnt_hsy   ;
    wire              end_cnt_hsy   ;
    wire              add_cnt_vsy   ;
    wire              end_cnt_vsy   ;
    reg [ 7:0]        cnt0          ;
    wire              add_cnt0      ;
    wire              end_cnt0      ;
    reg  [15:0]       cnt1          ;
    wire              add_cnt1      ;
    wire              end_cnt1      ;


    assign clk       = clk_50m              ;
    assign lcd_dclk  = ~ clk_50m            ;
    assign lcd_de    = hsync_de & vsync_de  ;
    
    


    always @ (posedge clk or negedge rst_n)begin
            if(!rst_n)begin
                cnt_hsy <= 0;
            end
            else if(add_cnt_hsy)begin
                if(end_cnt_hsy)
                    cnt_hsy <= 0;
                else
                    cnt_hsy <= cnt_hsy + 1;
            end
    end
    
    assign add_cnt_hsy = 1;
    assign end_cnt_hsy = add_cnt_hsy && cnt_hsy == LINE_PR -1;
    

    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt_vsy <= 0;
        end
        else if(add_cnt_vsy)begin
            if(end_cnt_vsy)
                cnt_vsy <= 0;
            else
                cnt_vsy <= cnt_vsy + 1;
        end
    end
   
    assign add_cnt_vsy = end_cnt_hsy;
    assign end_cnt_vsy = add_cnt_vsy && cnt_vsy == FRAME_PER - 1;
	 

    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            lcd_hsync <= 1'b0 ;
        end
        else if(end_cnt_hsy)begin
            lcd_hsync <= 1'b0;
        end
        else if(add_cnt_hsy && cnt_hsy == H_SYNC-1 )begin
            lcd_hsync <= 1'b1;
        end
    end

    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            hsync_de <= 1'b0;
        end
        else if(add_cnt_hsy && cnt_hsy == HDE_START-1)begin
            hsync_de <= 1'b1;
        end
        else if(add_cnt_hsy && cnt_hsy == HDE_END-1)begin
            hsync_de <= 1'b0;
        end
    end

	 
    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            lcd_vsync <= 1'b0 ;
        end
		  else if(add_cnt_vsy && cnt_vsy == V_SYNC-1 )begin
            lcd_vsync <= 1'b1;
        end
        else if(end_cnt_vsy)begin
            lcd_vsync <= 1'b0;
        end

    end

	 
    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            vsync_de <= 1'b0;
        end
        else if(add_cnt_vsy && cnt_vsy == VDE_START-1)begin
            vsync_de <= 1'b1;
        end
        else if(add_cnt_vsy && cnt_vsy ==VDE_END-1)begin
            vsync_de <= 1'b0;
        end
    end


    assign   display_area = hsync_de && vsync_de;


    assign   blue_area = (cnt_hsy >= HDE_START + 400-h) && (cnt_hsy<HDE_START+400+h) && 
                         (cnt_vsy >= VDE_START + 240-v) && (cnt_vsy<VDE_START+240+v) ; 

                     

        always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            h<=1;
        end
        else if(end_cnt_vsy && h<300)begin
            h<=h+2;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            v<=1;
        end
        else if(end_cnt_vsy && v<200)begin
            v<=v+1;
        end
    end


    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
				lcd_rgb <= 0;
        end
        else if(display_area)begin
            if(blue_area)begin
                lcd_rgb <= 24'h00_00_ff ;
            end
            else begin
                lcd_rgb <= 24'hff_ff_ff ;
            end
        end
        else begin
            lcd_rgb <= 0;
        end
    end



endmodule

